@@ -1,11 +1,8 @@
 from collections.abc import Iterator
-from typing import Any, Protocol, TypeAlias, runtime_checkable
+from typing import Protocol, TypeAlias, runtime_checkable
 
-from narwhals.expr import Expr
-from narwhals.series import Series
-
-# Base type for single expressions
-IntoExpr: TypeAlias = Expr | str | Series[Any]
+import anndata as ad
+from narwhals.typing import IntoExpr
 
 # Single predicate can be IntoExpr or list[bool]
 SinglePredicate: TypeAlias = IntoExpr | list[bool]
@@ -20,3 +17,7 @@ class PredicatesCollection(Protocol):
 
 # Final recursive type
 Predicates: TypeAlias = SinglePredicate | PredicatesCollection
+
+
+GroupNames: TypeAlias = tuple[str, ...]
+GroupBy: TypeAlias = ad.AnnData | tuple[GroupNames, ad.AnnData] | tuple[GroupNames, GroupNames, ad.AnnData]
